"""Provides the Base class and various mixins."""

from inspect import isclass
from sqlalchemy import Column, String, Integer, inspect
from sqlalchemy.ext.declarative import declarative_base
from .engine import engine
from .session import Session


class _Base:
    """The base class for declarative."""

    id = Column(Integer, primary_key=True)

    def __str__(self):
        return f'{getattr(self, "name", self.__class__.__name__)} (#{self.id})'

    def __repr__(self):
        res = f'{self.__class__.__name__} ('
        strings = []
        for name in inspect(self.__class__).columns.keys():
            strings.append(f'{name}={getattr(self, name)}')
        return res + ', '.join(strings) + ')'

    @classmethod
    def count(cls):
        return Session.query(cls).count()

    @classmethod
    def query(cls, *args, **kwargs):
        """Perform a query against this class."""
        return Session.query(cls).filter(*args).filter_by(**kwargs)

    @classmethod
    def first(cls):
        return cls.query().first()

    @classmethod
    def join(cls, *args, **kwargs):
        """Return a query object with a join."""
        return Session.query(cls).join(*args, **kwargs)

    @classmethod
    def get(cls, id):
        """Return a row with the exact id."""
        return Session.query(cls).get(id)

    @classmethod
    def number_of_objects(cls):
        """Return the number of objects in the database."""
        n = 0
        for base in cls._decl_class_registry.values():
            if isclass(base):
                n += base.count()
        return n


Base = declarative_base(cls=_Base, bind=engine)


class NameMixin:
    name = Column(String(50), nullable=True)


class NumberMixin:
    """Add a phone number."""

    number = Column(String(15), nullable=True)

    @classmethod
    def from_number(cls, number):
        """Get an instance which has the exact number."""
        return cls.query(number=number)
