"""Provides the classes needed for menus."""

import logging
import os.path
from glob import glob
from bs4 import BeautifulSoup
from attr import attrs, attrib, Factory
from . import application
from .ui import message

logger = logging.getLogger(__name__)
menus = {}

menus_dir = os.path.join(os.path.dirname(__file__), 'menus.xml')

NoneType = type(None)


@attrs
class Menu:
    """A menu containing MenuItem instances."""

    id = attrib()
    title = attrib()
    items = attrib(default=Factory(list), init=False)

    def add_item(self, item):
        self.items.append(item)
        item.parent = self

    @classmethod
    def from_soup(cls, soup):
        menu = cls(soup.get('id'), soup.get('title'))
        for data in soup.find_all('item'):
            item = MenuItem(
                data.get('id'),
                data.text,
                data.get('type'),
                data.get('value')
            )
            menu.add_item(item)
        return menu


@attrs
class MenuItem:
    """An item in a menu."""

    id = attrib()
    text = attrib()
    type = attrib()
    value = attrib()
    function = attrib(default=Factory(NoneType), init=False)
    menu = attrib(default=Factory(NoneType), init=False)
    parent = attrib(default=Factory(NoneType), init=False)


def load_menus():
    """Load menus from menus_dir."""
    items = []
    logger.info('Loading menus from %s.', menus_dir)
    for name in glob(os.path.join(menus_dir, '*.xml')):
        logger.info('Loading %s...', name)
        with open(name, 'r') as f:
            soup = BeautifulSoup(f.read(), 'html.parser')
            for top in soup.find_all('menu'):
                menu = Menu.from_soup(top)
                if menu.id in menus:
                    raise RuntimeError('Duplicated menu id %s.' % menu.id)
                menus[menu.id] = menu
                if menu.id is None:
                    if application.main_menu is not None:
                        raise RuntimeError('Only one main menu allowed.')
                    application.main_menu = menu
                items.extend(menu.items)
    for item in items:
        if item.type == 'menu':
            if item.id not in menus:
                raise RuntimeError(
                    'Invalid menu id %s from item %s.' % (item.id, item.text)
                )
            item.menu = menus[item.menu]
    if application.main_menu is None:
        raise RuntimeError('No main menu specified.')


def activate_menu(menu):
    """Activate the provided menu."""
    if menu is None:
        application.menu = None
        message.set_text(
            'Call window. Enter a number to call or press the call key to '
            'activate the main menu.', interrupt=False
        )
        return
    title = menu.title
    items = len(menu.items)
    if items == 1:
        text = 'item'
    else:
        text = 'items'
    message.set_text(f'{title.title()} with {items} {text}.')
    application.menu = menu
    application.menu_index = None
