"""The user interface stuff."""

import pyttsx3 as tts
from attr import attrs, attrib, Factory
from urwid import Text, Filler, MainLoop, ExitMainLoop
from . import application, menus
from .db import Settings, Contact

engine = tts.init()
engine.startLoop(False)

tts_interval = 0.01


def idle(loop, data):
    engine.iterate()
    loop.set_alarm_in(tts_interval, idle)


class Message(Text):
    def set_text(self, text, interrupt=True):
        super().set_text(text)
        if interrupt:
            engine.stop()
        engine.say(text)


@attrs
class Keyboard:
    """Hold keyboard actions."""

    locked = attrib(default=Factory(bool))
    lock_pressed = attrib(
        default=Factory(lambda: Settings.active().unlock_keypresses)
    )
    last_key = attrib(default=Factory(lambda: application.lock_key))
    functions = attrib(default=Factory(dict))
    when_locked = attrib(default=Factory(list))
    number = attrib(default=Factory(str))

    def key(self, name, locked=False):
        """Decorate a function. If locked is True allow the key even when the
        screen is locked."""
        def inner(func):
            assert name not in self.functions
            self.functions[name] = func
            if locked:
                self.when_locked.append(name)
            return func
        return inner


message = Message('Loading...')
filler = Filler(message, 'top')
keyboard = Keyboard()


@keyboard.key('.')
def do_menu_or_call():
    """Call key."""
    if application.menu is None:
        if keyboard.number:
            contact = Contact.from_number(keyboard.number)
            if contact is None:
                name = keyboard.number
            else:
                name = f'{contact.name} ({keyboard.number})'
            message.set_text(f'Calling {name}...')
            keyboard.number = ''
        else:
            menus.activate_menu(application.main_menu)
    else:
        menus.activate_menu(None)


@keyboard.key('q', locked=True)
def do_quit():
    """Quit the OS."""
    raise ExitMainLoop()


@keyboard.key(application.lock_key, locked=True)
def do_lock():
    """Lock or unlock after 3 consecutive presses."""
    if keyboard.last_key == application.lock_key:
        keyboard.lock_pressed += 1
        if keyboard.locked:
            presses = Settings.active().unlock_keypresses
        else:
            presses = 0
        if keyboard.lock_pressed >= presses:
            keyboard.lock_pressed = 0
            keyboard.locked = not keyboard.locked
            message.set_text(
                f'Phone {"locked" if keyboard.locked else "unlocked"}.'
            )
            if not keyboard.locked:
                menus.activate_menu(application.menu)
    else:
        keyboard.lock_pressed = 1


def on_key(key):
        if not keyboard.locked or key in keyboard.when_locked:
            if key in keyboard.functions:
                keyboard.functions[key]()
            else:
                message.set_text(f'You pressed {key}.')
        keyboard.last_key = key


loop = MainLoop(filler, unhandled_input=on_key)
loop.set_alarm_in(tts_interval, idle)
